@@ -163,7 +163,7 @@ _KOG_TABLE = dict(A = "RNA processing and mofiication",
 import gc
 from collections.abc import Iterable
 class FunctionalEnrichment(object):    
-    def __init__(self,sms, EGGS, keys, col_key = 'best_OG_cat', delimiter = '', align_thr = 0.1, limit_reference = False, n_top = 0):
+    def __init__(self,sms, DFS, col_key, keys, delimiter = '', align_thr = 0.1, limit_reference = False, n_top = 0):
         """Performs functional enrichment analysis on gene pairs enriched
         in mapped cell types using functional annotations output by Eggnog.
         
@@ -171,20 +171,25 @@ class FunctionalEnrichment(object):
         ----------
         sms - list or tuple of SAMAP objects
         
-        EGGS - list or tuple of pandas.DataFrame Eggnog annotation (one per species present in the input SAMAP objects)
+        DFS - list or tuple of pandas.DataFrame functional annotations (one per species present in the input SAMAP objects)
+        
+        col_key - str
+            The column name with functional annotations in the annotation DataFrames.
         
         keys - list or tuple of column keys from `.adata.obs` DataFrames (one per species present in the input SAMAP objects)
             Cell type mappings will be computed between these annotation vectors.
-            
-        col_key - str, optional, default 'best_OG_cat'
-            The column name with functional annotations in the Eggnog annotation DataFrames.
+                        
+        delimiter - str, optional, default ''
+            Some transcripts may have multiple functional annotations (e.g. GO terms or KOG terms) separated by
+            a delimiter. For KOG terms, this is typically no delimiter (''). For GO terms, this is usually a comma
+            (',').
             
         align_thr - float, optional, default 0.1
             The alignment score below which to filter out cell type mappings
             
         limit_reference - bool, optional, default False
             If True, limits the background set of genes to include only those that are enriched in any cell type mappings
-            If False, the background set of genes will include all genes present in Eggnog dataframes.
+            If False, the background set of genes will include all genes present in the input dataframes.
             
         n_top: int, optional, default 0
             If `n_top` is 0, average the alignment scores for all cells in a pair of clusters.
@@ -196,8 +201,8 @@ class FunctionalEnrichment(object):
         # get dictionary of sam objects
         if not isinstance(sms,Iterable):
             sms = [sms]
-        if not isinstance(EGGS,Iterable):
-            EGGS = [EGGS]
+        if not isinstance(DFS,Iterable):
+            DFS = [DFS]
         if not isinstance(keys,Iterable):
             keys = [keys]
 
@@ -214,20 +219,20 @@ class FunctionalEnrichment(object):
             
         # figure out which species corresponds to which EGGNOG table
         keys2 = {}
-        for i in range(len(EGGS)):
-            EGGS[i] = EGGS[i].copy()
+        for i in range(len(DFS)):
+            DFS[i] = DFS[i].copy()
             
-            genes = q(EGGS[i].index)
+            genes = q(DFS[i].index)
             overlap=[]
             ks = list(SAMS.keys())
             for k in ks:
                 overlap.append(np.in1d(genes,['_'.join(x.split('_')[1:]) for x in SAMS[k].adata.var_names]).mean())
             k = ks[np.array(overlap).argmax()]
-            EGGS[i].index = k+'_'+EGGS[i].index
+            DFS[i].index = k+'_'+DFS[i].index
             keys2[k] = keys[i]
         keys = keys2    
-        # concatenate EGGS
-        A = pd.concat(EGGS,axis=0)
+        # concatenate DFS
+        A = pd.concat(DFS,axis=0)
         RES = pd.DataFrame(A[col_key])
         RES.columns=['GO']    
         RES = RES[(q(RES.values.flatten())!='nan')]
@@ -273,7 +278,10 @@ class FunctionalEnrichment(object):
         
         self.DICT = {}
         for c in gene_pairs.columns:
-            self.DICT[c] = q(gene_pairs[c].values.flatten()[q(gene_pairs[c].values.flatten())!='nan'])
+            x = q(gene_pairs[c].values.flatten()).astype('str')
+            ff = x!='nan'
+            if ff.sum()>0:
+                self.DICT[c] = x[ff]
 
         if limit_reference:
             all_genes = np.unique(np.concatenate(substr(np.concatenate(list(self.DICT.values())),';')))
@@ -283,12 +291,30 @@ class FunctionalEnrichment(object):
         for d in GENE_SETS.keys():
             GENE_SETS[d] = GENE_SETS[d][np.in1d(GENE_SETS[d],all_genes)]
 
-
+        self.gene_pairs = gene_pairs
         self.CAT_NAMES = np.unique(q(RES['GO']))
         self.GENE_SETS = GENE_SETS
         self.RES = RES
         
     def calculate_enrichment(self,verbose=False):
+        """ Calculates the functional enrichment.
+        
+        Parameters
+        ----------
+        verbose - bool, optional, default False
+            If False, function does not log progress to output console.
+            
+        Returns
+        -------
+        ENRICHMENT_SCORES - pandas.DataFrame (cell types x function categories)
+            Enrichment scores (-log10 p-value) for each function in each cell type.
+        
+        NUM_ENRICHED_GENES - pandas.DataFrame (cell types x function categories)
+            Number of enriched genes for each function in each cell type.        
+        
+        ENRICHED_GENES - pandas.DataFrame (cell types x function categories)
+            The IDs of enriched genes for each function in each cell type.
+        """
         DICT = self.DICT
         RES = self.RES
         CAT_NAMES = self.CAT_NAMES
@@ -339,7 +365,7 @@ class FunctionalEnrichment(object):
                     HMe[Z[gt].values.flatten(),ii] = lens
                     HMg[Z[gt].values.flatten(),ii] = [';'.join(np.unique(x.split(';'))) for x in result['genes'].values]
 
-        CAT_NAMES = [_KOG_TABLE[x] for x in CAT_NAMES]
+        #CAT_NAMES = [_KOG_TABLE[x] for x in CAT_NAMES]
         SC = pd.DataFrame(data = HM,index=CAT_NAMES,columns=all_nodes).T
         SCe = pd.DataFrame(data = HMe,index=CAT_NAMES,columns=all_nodes).T
         SCg = pd.DataFrame(data = HMg,index=CAT_NAMES,columns=all_nodes).T
@@ -349,9 +375,28 @@ class FunctionalEnrichment(object):
         self.NUM_ENRICHED_GENES = SCe
         self.ENRICHED_GENES = SCg
         
-        return SC,SCe,SCg
+        return self.ENRICHMENT_SCORES,self.NUM_ENRICHED_GENES,self.ENRICHED_GENES
     
-    def plot_enrichment(self,cell_types = [], pval_thr=2,msize = 50):
+    def plot_enrichment(self,cell_types = [], pval_thr=2.0,msize = 50):
+        """Create a plot summarizing the functional enrichment analysis.
+        
+        Parameters
+        ----------
+        cell_types - list, default []
+            A list of cell types for which enrichment scores will be plotted. If empty (default),
+            all cell types will be plotted.
+            
+        pval_thr - float, default 2.0
+            -log10 p-values < 2.0 will be filtered from the plot.
+            
+        msize - float, default 50
+            The marker size in pixels for the dot plot.
+        
+        Returns
+        -------
+        fig - matplotlib.pyplot.Figure
+        ax - matplotlib.pyplot.Axes
+        """
         import colorsys
         import seaborn as sns
         import matplotlib
@@ -602,9 +647,9 @@ class GenePairFinder(object):
         assert n2 in q(self.s2.adata.obs[self.k2])
 
         if True:
-            m = self.find_link_genes_avg(n1, n2, w1t=w1t, w2t=w2t, expr_thr=0.05)
+            m = self._find_link_genes_avg(n1, n2, w1t=w1t, w2t=w2t, expr_thr=0.05)
         else:
-            m = self.find_link_genes(
+            m = self._find_link_genes(
                 n1, n2, w1t=w1t, w2t=w2t, n_pairs=500, expr_thr=0.05
             )
 
@@ -629,7 +674,7 @@ class GenePairFinder(object):
         G2 = G2[np.sort(ix2)]
         return G, G1, G2
 
-    def find_link_genes_avg(self, c1, c2, w1t=0.35, w2t=0.35, expr_thr=0.05):
+    def _find_link_genes_avg(self, c1, c2, w1t=0.35, w2t=0.35, expr_thr=0.05):
         mu1 = self.mu1
         std1 = self.v1
         mu2 = self.mu2
@@ -678,7 +723,7 @@ class GenePairFinder(object):
         w2[w2 > 0] = 1
         return val * w1 * w2 * min_expr
 
-    def find_link_genes(
+    def _find_link_genes(
         self, c1, c2, w1t=0.35, w2t=0.35, knn=False, n_pairs=250, expr_thr=0.05
     ):
         mu1 = self.mu1
@@ -741,7 +786,25 @@ class GenePairFinder(object):
         return mu
 
 
-def find_cluster_markers(sam, key, layer=None, inplace=True):
+def find_cluster_markers(sam, key, inplace=True):
+    """ Finds differentially expressed genes for provided cell type labels.
+    
+    Parameters
+    ----------
+    sam - SAM object
+    
+    key - str
+        Column in `sam.adata.obs` for which to identifying differentially expressed genes.
+        
+    inplace - bool, optional, default True
+        If True, deposits enrichment scores in `sam.adata.varm[f'{key}_scores']`
+        and p-values in `sam.adata.varm[f'{key}_pvals']`.
+        
+        Otherwise, returns three pandas.DataFrame objects (genes x clusters).
+            NAMES - the gene names
+            PVALS - the p-values
+            SCORES - the enrichment scores
+    """
     sam.adata.raw = sam.adata_raw
     a,c = np.unique(q(sam.adata.obs[key]),return_counts=True)
     t = a[c==1]
@@ -753,7 +816,7 @@ def find_cluster_markers(sam, key, layer=None, inplace=True):
         method="wilcoxon",
         n_genes=sam.adata.shape[1],
         use_raw=True,
-        layer=layer,
+        layer=None,
     )
     sam.adata.uns['rank_genes_groups'] = adata.uns['rank_genes_groups']
     
@@ -798,7 +861,10 @@ def find_cluster_markers(sam, key, layer=None, inplace=True):
 
 
 def ParalogSubstitutions(sm, ortholog_pairs, paralog_pairs=None, psub_thr = 0.3):
-    """Identify paralog substitutions
+    """Identify paralog substitutions. 
+    
+    For all genes in `ortholog_pairs` and `paralog_pairs`, this function expects the genes to
+    be prepended with their corresponding species IDs (i.e. `sm.id1` or `sm.id2`).
     
     Parameters
     ----------
@@ -811,19 +877,81 @@ def ParalogSubstitutions(sm, ortholog_pairs, paralog_pairs=None, psub_thr = 0.3)
         Note that this would essentially result in the more generic 'homolog substitutions' rather
         than paralog substitutions.
         
+        The paralogs can be either cross-species, within-species, or a mix of both. 
+        
+    psub_thr - float, optional, default 0.3
+        Threshold for correlation difference between paralog pairs and ortholog pairs.
+        Paralog pairs that do not have greater than `psub_thr` correlation than their 
+        corresponding ortholog pairs are filtered out.
+        
     Returns
     -------
     RES - pandas.DataFrame
         A table of paralog substitutions.
         
     """
+    if paralog_pairs is not None:
+        ids1 = np.array([x.split('_')[0] for x in paralog_pairs[:,0]])
+        ids2 = np.array([x.split('_')[0] for x in paralog_pairs[:,1]])
+        ix = np.where(ids1==ids2)[0]
+        ixnot = np.where(ids1!=ids2)[0]
+        
+        if ix.size > 0:
+            pps = paralog_pairs[ix]
+            
+            ZZ1 = {}
+            ZZ2 = {}
+            for i in range(pps.shape[0]):    
+                L = ZZ1.get(pps[i,0],[])
+                L.append(pps[i,1])
+                ZZ1[pps[i,0]]=L
+
+                L = ZZ2.get(pps[i,1],[])
+                L.append(pps[i,0])
+                ZZ2[pps[i,1]]=L      
+
+            keys = list(ZZ1.keys())
+            for k in keys:
+                L = ZZ2.get(k,[])
+                L.extend(ZZ1[k])
+                ZZ2[k] = list(np.unique(L))
+
+            ZZ = ZZ2
+
+            L1=[]
+            L2=[]
+            for i in range(ortholog_pairs.shape[0]):
+                try:
+                    x = ZZ[ortholog_pairs[i,0]]
+                except:
+                    x = []
+                L1.extend([ortholog_pairs[i,1]]*len(x))
+                L2.extend(x)
+
+                try:
+                    x = ZZ[ortholog_pairs[i,1]]
+                except:
+                    x = []
+                L1.extend([ortholog_pairs[i,0]]*len(x))
+                L2.extend(x)        
+
+            L = np.vstack((L2,L1)).T
+            pps = np.unique(np.sort(L,axis=1),axis=0)
+            
+            paralog_pairs = np.unique(np.sort(np.vstack((pps,paralog_pairs[ixnot])),axis=1),axis=0)
+        
+        
     smp = sm.samap
     
     gnnm = smp.adata.uns["homology_graph_reweighted"]
     gn = sm.gn
     
+    ortholog_pairs = ortholog_pairs[np.logical_and(np.in1d(ortholog_pairs[:,0],gn),np.in1d(ortholog_pairs[:,1],gn))]
     if paralog_pairs is None:
         paralog_pairs = gn[np.vstack(smp.adata.uns["homology_graph"].nonzero()).T]
+    else:
+        paralog_pairs = paralog_pairs[np.logical_and(np.in1d(paralog_pairs[:,0],gn),np.in1d(paralog_pairs[:,1],gn))]
+        
     paralog_pairs = paralog_pairs[
         np.in1d(to_vn(paralog_pairs), np.append(to_vn(ortholog_pairs),to_vn(ortholog_pairs[:,::-1])), invert=True)
     ]
@@ -877,7 +1005,7 @@ def ParalogSubstitutions(sm, ortholog_pairs, paralog_pairs=None, psub_thr = 0.3)
     return RES
 
 
-def convert_eggnog_to_homologs(sm, A, B, taxon=2759):
+def convert_eggnog_to_homologs(sm, A, B, og_key = 'eggNOG_OGs', taxon=2759):
     """Gets an n x 2 array of homologs at some taxonomic level based on Eggnog results.
     
     Parameters
@@ -887,7 +1015,10 @@ def convert_eggnog_to_homologs(sm, A, B, taxon=2759):
     A: pandas.DataFrame, Eggnog output table
     
     B: pandas.DataFrame, Eggnog output table
-    
+
+    og_key: str, optional, default 'eggNOG_OGs'
+        The column name of the orthology group mapping results in the Eggnog output table.
+
     taxon: int, optional, default 2759
         Taxonomic ID corresponding to the level at which genes with overlapping orthology groups
         will be considered homologs. Defaults to the Eukaryotic level.
@@ -910,22 +1041,22 @@ def convert_eggnog_to_homologs(sm, A, B, taxon=2759):
     gn = q(smp.adata.uns["homology_gene_names"])
     A = A[np.in1d(q(A.index), gn)]
 
-    orthology_groups = A["18"]
+    orthology_groups = A[og_key]
     og = q(orthology_groups)
     x = np.unique(",".join(og).split(","))
     D = pd.DataFrame(data=np.arange(x.size)[None, :], columns=x)
 
     for i in range(og.size):
         n = orthology_groups[i].split(",")
-        taxa = substr(n, "@", 1)
-        if (taxa == "2759").sum() > 1:
+        taxa = substr(substr(n, "@", 1),'|',0)
+        if (taxa == "2759").sum() > 1 and taxon == '2759':
             og[i] = ""
         else:
             og[i] = "".join(np.array(n)[taxa == taxon])
 
-    A["18"] = og
+    A[og_key] = og
 
-    og = q(A["18"][gn])
+    og = q(A[og_key][gn])
     og[og == "nan"] = ""
 
     X = []
@@ -1003,7 +1134,7 @@ def CellTypeTriangles(sms,keys, align_thr=0.1):
 
         smp.adata.obs["triangle_{}{}".format(x, y)] = pd.Categorical(cl1)
 
-        _, ax, bx, CSIMt = compute_csim(smp, key="triangle_{}{}".format(x, y))
+        _, ax, bx, CSIMt = _compute_csim(smp, key="triangle_{}{}".format(x, y))
         pairsi = np.vstack(CSIMt.nonzero()).T
         pairs = np.vstack((ax[pairsi[:, 0]], bx[pairsi[:, 1]])).T
         X.append(pairs)
@@ -1262,7 +1393,7 @@ def SubstitutionTriangles(sms,orths,keys=None,compute_markers=True,corr_thr=0.3,
     return FINAL
 
 
-def compute_csim(sam3, key, X=None, n_top = 0):
+def _compute_csim(sam3, key, X=None, n_top = 0):
     if n_top ==0:
         n_top = 100000000
         
@@ -1430,7 +1561,7 @@ def get_mapping_scores(sm, key1, key2, n_top = 0):
     )
 
     samap.adata.obs["{};{}_mapping_scores".format(key1,key2)] = pd.Categorical(cl)
-    _, clu1, clu2, CSIMth = compute_csim(samap, "{};{}_mapping_scores".format(key1,key2), n_top = n_top)
+    _, clu1, clu2, CSIMth = _compute_csim(samap, "{};{}_mapping_scores".format(key1,key2), n_top = n_top)
 
     A = pd.DataFrame(data=CSIMth, index=clu1, columns=clu2)
     i = np.argsort(-A.values.max(0).flatten())
